﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Animations;
using UnityEngine.Playables;

public class BackAndForthAnimation : MonoBehaviour
{
    public AnimationClip _clip;

    PlayableGraph _playableGraph;
    AnimationClipPlayable _clipPlayable;

    public bool open = false;
    Animator _animator;
    Animation _animationLegacy;
    AnimationState _animationStateLegacy;
    public float speed = 1;

    void Start()
    {
        _animator = GetComponent<Animator>();
        _animationLegacy = GetComponent<Animation>();
        if (_animator != null && _animationLegacy != null)
        {
            Debug.LogError("Shouldn't have both kinds of animator on this object! : " + this.name, this.gameObject);
        }

        if (_animator != null)
        {
            setUpWithAnimator();
        }
        else if (_animationLegacy != null)
        {
            setUpWithLegacyAnimation();
        }
    }

    public void ToggleOpen()
    {
        open = !open;
    }

    public void SetOpen(bool shouldOpen)
    {
        open = shouldOpen;
    }

    void LateUpdate()
    {
        if (_animator != null)
        {
            updateAnimator();
        }
        else
        {
            updateLegacyAnimation();
        }

    }

    void setUpWithLegacyAnimation()
    {
        if (_clip != null)
        {
            _animationLegacy.AddClip(_clip, _clip.name);
            _animationLegacy.clip = _clip;

        }
        _animationStateLegacy = _animationLegacy[_animationLegacy.clip.name];
        _animationStateLegacy.wrapMode = WrapMode.Clamp;
        _animationStateLegacy.speed = 0;
        _animationLegacy.Play();
    }

    void updateLegacyAnimation()
    {
        float clipLength = _animationStateLegacy.length;
        float targetTime = open ? clipLength : 0;
        _animationStateLegacy.time = Mathf.MoveTowards(_animationStateLegacy.time, targetTime, Time.deltaTime * speed);
    }

    void setUpWithAnimator()
    {
        _playableGraph = PlayableGraph.Create();

        _playableGraph.SetTimeUpdateMode(DirectorUpdateMode.GameTime);

        var playableOutput = AnimationPlayableOutput.Create(_playableGraph, "Animation", _animator);

        // Wrap the clip in a playable

        _clipPlayable = AnimationClipPlayable.Create(_playableGraph, _clip);
        _clipPlayable.Pause();

        // Connect the Playable to an output

        playableOutput.SetSourcePlayable(_clipPlayable);

        // Plays the Graph.

        _playableGraph.Play();
    }

    void updateAnimator()
    {
        double finalTime = _clipPlayable.GetTime() + (open ? speed : -speed)  * Time.deltaTime;

        float frameHz = 0.5f / _clip.frameRate;
        if (finalTime >= _clip.length - frameHz)
        {
            finalTime = _clip.length - frameHz;
        }
        else if (finalTime < 0)
        {
            finalTime = 0;
        }

        _clipPlayable.SetTime(finalTime);

    }
    
    private void OnDestroy()
    {
        if (_playableGraph.IsValid())
        {
            _playableGraph.Destroy();
        }
    }
}
